#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Interpreter version: python 2.7
#
# Imports =====================================================================
from title_detector import get_title_tags
from place_detector import get_place_tags
from author_detector import get_author_tags
from language_detector import get_lang_tags
from keyword_detector import get_keyword_tags
from annotation_detector import get_annotation_tags
from creation_date_detector import get_creation_date_tags
